import configparser
import os

from digitaltwins import Auth
from digitaltwins import IRODS
from digitaltwins import MetadataQuerier
from digitaltwins.utils.config_loader import ConfigLoader

from irods.session import iRODSSession


class Downloader(object):
    def __init__(self, config_file):
        self._configs = configparser.ConfigParser()
        self._configs.read(config_file)

        self._save_dir = "./tmp"

        self._irods = IRODS(self._configs)

    def execute(self, dataset_name=None, save_dir=None):
        if dataset_name is None:
            raise ValueError("Dataset not specified")
        if save_dir is None:
            save_dir = self._save_dir

        self._irods.download(dataset_name, save_dir)

    def _download_collection(self, session, collection_path, save_dir):
        dataset = session.collections.get(collection_path)
        save_dir = save_dir.joinpath(dataset.name)
        os.makedirs(save_dir, exist_ok=True)

        for obj in (dataset.data_objects):
            session.data_objects.get(obj.path, os.path.join(save_dir, obj.name))

        if dataset.subcollections:
            for subcollection in dataset.subcollections:
                self._download_collection(session, subcollection.path, save_dir)
        else:
            return

    # def __init__(self, data_storage_config, gen3_config=None, data_storage_type="pacs"):
    #     """
    #     Constructor
    #
    #     :param data_storage_config: Path to the data storage (PACS or iRods) configuration file (json)
    #     :type data_storage_config: string
    #     :param gen3_config: (Optional)
    #     :type gen3_config: Path to the Gen3 configuration file (json)
    #     """
    #     self._data_storage_configs = ConfigLoader.load_from_json(data_storage_config)
    #
    #     self._data_storage = self._data_storage_configs.get("storage")
    #     if data_storage_type:
    #         self._data_storage = data_storage_type
    #     else:
    #         self._data_storage = self._data_storage_configs.get("storage")
    #
    #     if gen3_config:
    #         self._gen3_config = ConfigLoader.load_from_json(gen3_config)
    #         self._gen3_endpoint = self._gen3_config.get("gen3_endpoint")
    #         self._gen3_cred_file = self._gen3_config.get("gen3_cred_file")
    #         self._gen3_auth = Auth(self._gen3_endpoint, self._gen3_cred_file)
    #         self._gen3_queryer = MetadataQuerier(self._gen3_auth)
    #
    #     if self._data_storage == "pacs":
    #         self._pacs_ip = self._data_storage_configs.get("pacs_ip")
    #         self._pacs_port = self._data_storage_configs.get("pacs_port")
    #         self._pacs_aec = self._data_storage_configs.get("pacs_aec")
    #         self._pacs_aet = self._data_storage_configs.get("pacs_aet")
    #
    #     elif self._data_storage == "irods":
    #         self._irods = IRODS(data_storage_config)
    #
    # def download_dataset(self, dataset_id, dest):
    #     """
    #     Downloading dataset (including data from PACS/iRods & metadata from Gen3) in SDS format
    #
    #     :param dataset_id: Dataset id/name on Gen3
    #     :type dataset_id: str
    #     :param dest: Path to the save folder
    #     :type dest: string
    #     """
    #     if self._data_storage == "pacs":
    #         # self._download(dataset_id)
    #         raise NotImplementedError("PACS: Storage location not implemented!")
    #     elif self._data_storage == "irods":
    #         self._irods.download_data(data=dataset_id, save_dir=dest)

    # def _download(self, dataset_id):
    #     gen3_query_string = """
    #     {
    #         experiment(submitter_id: "%s"){
    #             cases{
    #                 id
    #                 submitter_id,
    #                 subject_id,
    #             }
    #         }
    #     }
    #     """ % dataset_id
    #
    #     results = self._queryer.graphql_query(gen3_query_string)
    #     datasets = results.get("experiment")
    #
    #     study_uuids = list()
    #     for dataset in datasets:
    #         studies = dataset.get("cases")
    #         for study in studies:
    #             study_id = study.get("subject_id")
    #             study_uuid = study_id.replace("sub-", '')
    #             study_uuids.append(study_uuid)
    #
    #     for study_uuid in study_uuids:
    #         pacs_query_settings = {
    #             "StudyInstanceUID": study_uuid
    #         }
    #         pypacs.move_files(server_ip=self._pacs_ip,
    #                           server_port=self._pacs_port,
    #                           aec=self._pacs_aec,
    #                           aet=self._pacs_aet,
    #                           query_settings=pacs_query_settings)



import configparser

from gen3.submission import Gen3Submission
from gen3.auth import Gen3Auth


def get_project_id(program, project):
    project_id = program + '-' + project
    return project_id


class MetadataQuerier(object):
    """
    Class for querying Gen3.
    Also accepts queries in GraphQL syntax.
    """
    def __init__(self, config_file):
        """
        Constructor

        :param auth: Gen3 authentication object created by the Auth class
        :type auth: object
        """
        self._configs = configparser.ConfigParser()
        self._configs.read(config_file)

        self._endpoint = self._configs["gen3"].get("endpoint")
        self._cred_file = self._configs["gen3"].get("cred_file")
        self._program = self._configs["gen3"].get("program")
        self._project = self._configs["gen3"].get("project")

        self._auth = Gen3Auth(self._endpoint, self._cred_file)

        self._querier = Gen3Submission(self._auth)

    def get_program(self):
        return self._program

    def get_project(self):
        return self._project

    def graphql_query(self, query_string, variables=None):
        """
        Sending a GraphQL query to Gen3

        :param query_string: query in GraphQL syntax
        :type query_string: string
        :param variables: query variables (optional)
        :type variables: dict
        :return: query response
        :rtype: dict
        """
        response = self._querier.query(query_string, variables).get("data")

        return response

    def get_programs_all(self):
        """
        Getting all programs that the user have access to

        :return: List of programs
        :rtype: list
        """
        response = self._querier.get_programs()

        programs = list()
        for resp in response.get("links"):
            program = resp.split('/')[-1]
            programs.append(program)

        return programs

    def get_projects_by_program(self, program):
        """
        Getting the projects by program name

        :param program: Name of a Gen3 program
        :type program: str
        :return: List of projects
        :rtype: list
        """
        response = self._querier.get_projects(program)

        projects = list()
        for resp in response.get("links"):
            project = resp.split('/')[-1]
            projects.append(project)

        return projects

    def get_datasets(self, program=None, project=None):
        if program is None:
            program = self._program
        if project is None:
            project = self._project

        project_id = get_project_id(program, project)

        query_string = f"""
        {{
          experiment (project_id: "{project_id}"){{
              id,
              submitter_id
          }}
        }}
        """
        response = self.graphql_query(query_string).get("experiment")


        datasets = list()
        for element in response:
            submitter_id = element.get("submitter_id")
            datasets.append(submitter_id)

        return datasets

    def get_subjects(self, dataset_id, program=None, project=None):
        if program is None:
            program = self._program
        if project is None:
            project = self._project

        project_id = get_project_id(program, project)

        query_string = f"""
        {{
          experiment(project_id: "{project_id}", submitter_id: "{dataset_id}"){{
            cases{{
              subject_id
            }}
          }}
        }}
        """
        response = self.graphql_query(query_string)
        experiment = response.get("experiment")[0]
        cases = experiment.get("cases")
        return cases

    def get_dataset_descriptions(self, dataset_id, program=None, project=None):
        if program is None:
            program = self._program
        if project is None:
            project = self._project

        project_id = get_project_id(program, project)
        query_string = f"""
        {{
          experiment(project_id: "{project_id}", submitter_id: "{dataset_id}"){{
            dataset_descriptions{{
              metadata_version,
              dataset_type,
              title,
              subtitle,
              keywords,
              funding,
              acknowledgments,
              study_purpose,
              study_data_collection,
              study_primary_conclusion,
              study_organ_system,
              study_approach,
              study_technique,
              study_collection_title,
              contributor_name,
              contributor_orcid,
              contributor_affiliation,
              contributor_role,
              identifier_description,
              relation_type,
              identifier,
              identifier_type,
              number_of_subjects,
              number_of_samples,
              dataset_type,
              title,
              subtitle,
              keywords,
              funding,
              acknowledgments,
              study_purpose,
              study_data_collection,
              study_primary_conclusion,
              study_organ_system,
              study_approach,
              study_technique,
              study_collection_title,
              contributor_name,
              contributor_orcid,
              contributor_affiliation,
              contributor_role,
              identifier_description,
              relation_type,
              identifier,
              identifier_type,
              number_of_subjects,
              number_of_samples
            }}
          }}
        }}
        """

        response = self.graphql_query(query_string)
        dataset = response.get("experiment")[0]
        dataset_descriptions = dataset.get("dataset_descriptions")

        return dataset_descriptions



    def get_node_records(self, node, program, project):
        """
        Getting all the records in a Gen3 node

        :param node: Name of the target node
        :type node: str
        :param program: program name
        :type program: str
        :param project: project name
        :type project: str
        :return: A list of records in dictionary format
        :rtype: list
        """
        response = self._querier.export_node(program, project, node, "json")
        data = response.get("data")
        return data





import configparser
from digitaltwins.irods.irods import IRODS


class Deleter(object):
    def __init__(self, config_file):
        self._configs = configparser.ConfigParser()
        self._configs.read(config_file)

        self._program = self._configs["gen3"].get("program")
        self._project = self._configs["gen3"].get("project")

    def execute(self, dataset_name):
        self._delete_metadata(dataset_name)
        self._delete_dataset(dataset_name)
        pass

    def _delete_metadata(self, dataset_name):
        pass

    def _delete_dataset(self, dataset_name):
        pass
